@@ -11,6 +11,7 @@
 #    available at https://github.com/cutright/DVHA-Stats
 
 
+from matplotlib.pyplot import boxplot
 import numpy as np
 from scipy import stats as scipy_stats
 from sklearn import linear_model
@@ -154,7 +155,16 @@ class Histogram:
 class MultiVariableRegression:
     """Multi-variable regression using scikit-learn"""
 
-    def __init__(self, X, y, saved_reg=None, var_names=None):
+    def __init__(
+        self,
+        X,
+        y,
+        saved_reg=None,
+        var_names=None,
+        y_var_name=None,
+        back_elim=False,
+        back_elim_p=0.05,
+    ):
         """Initialization of a MultiVariableRegression
 
         Parameters
@@ -167,10 +177,17 @@ class MultiVariableRegression:
             Optionally provide a previously calculated regression
         var_names : list, optional
             Optionally provide names of the variables
+        y_var_name : int, str, optional
+            Optionally provide name of the dependent variable
+        back_elim : bool
+            Automatically perform backward elimination if True
+        back_elim_p : float
+            p-value threshold for backward elimination
         """
 
         self.X = np.array(X)
         self.y = np.array(y)
+        self.y_var_name = y_var_name
 
         self.var_names = (
             list(range(self.X.shape[1])) if var_names is None else var_names
@@ -178,6 +195,9 @@ class MultiVariableRegression:
 
         self._do_fit(saved_reg=saved_reg)
 
+        if back_elim:
+            self.backward_elimination(back_elim_p)
+
     def __str__(self):
         """String representation of MultiVariableRegression object"""
 
@@ -185,12 +205,14 @@ class MultiVariableRegression:
 
         table.field_names = ["", "Coef", "Std. Err.", "t-value", "p-value"]
         for c in table.field_names[1:]:
-            table.align[c] = 'r'
-        data = {"": ["y-int"] + self.var_names,
-                "Coef": ["%0.3E" % v for v in self.coef],
-                "Std. Err.": ["%0.3E" % v for v in self.std_err],
-                "t-value": ["%0.3f" % v for v in self.t],
-                "p-value": ["%0.3f" % v for v in self.p]}
+            table.align[c] = "r"
+        data = {
+            "": ["y-int"] + self.var_names,
+            "Coef": ["%0.3E" % v for v in self.coef],
+            "Std. Err.": ["%0.3E" % v for v in self.std_err],
+            "t-value": ["%0.3f" % v for v in self.t],
+            "p-value": ["%0.3f" % v for v in self.p],
+        }
 
         for r in range(0, len(data[""])):
             table.add_row([data[c][r] for c in table.field_names])
@@ -201,7 +223,7 @@ class MultiVariableRegression:
             "MSE: %0.3f" % self.mse,
             "f-stat: %0.3f" % self.f_stat,
             "f-stat p-value: %0.3f" % self.f_p_value,
-            str(table)
+            str(table),
         ]
         return "\n".join(msg)
 
@@ -345,6 +367,24 @@ class MultiVariableRegression:
         """
         return np.concatenate(([self.y_intercept], self.slope))
 
+    def backward_elimination(self, p_value=0.05):
+        """Remove insignificant variables from regression
+
+        p_value : float
+            Iteratively remove the least significant variable until all
+            variables have p-values less than p_value or only one variable
+            remains.
+        """
+        finished = False
+        while not finished:
+            index = self.p[1:].argmax()  # ignore y-int
+            if len(self.var_names) > 1 and self.p[index + 1] > p_value:
+                self.X = np.delete(self.X, index, axis=1)
+                self.var_names.pop(index)
+                self._do_fit()
+            else:
+                finished = True
+
     @property
     def chart_data(self):
         """JSON compatible dict for chart generation
@@ -739,6 +779,65 @@ class HotellingT2:
         }
 
 
+class RiskAdjustedControlChart(ControlChart):
+    def __init__(
+        self,
+        X,
+        y,
+        std=3,
+        ucl_limit=None,
+        lcl_limit=None,
+        x=None,
+        saved_reg=None,
+        var_names=None,
+        reg_vars=None,
+        back_elim=False,
+        back_elim_p=0.05,
+    ):
+        """
+        Calculate control limits for a standard univariate Control Chart
+
+        Parameters
+        ----------
+        X : array-like
+                Independent data
+        y : list, np.ndarray
+            Input data (1-D)
+        std : int, float, optional
+            Number of standard deviations used to calculate if a y-value is
+            out-of-control.
+        ucl_limit : float, optional
+            Limit the upper control limit to this value
+        lcl_limit : float, optional
+            Limit the lower control limit to this value
+        saved_reg : MultiVariableRegression, optional
+            Optionally provide a previously calculated regression
+        var_names : list, optional
+            Optionally provide names of the variables
+        back_elim : bool
+            Automatically perform backward elimination if True
+        back_elim_p : float
+            p-value threshold for backward elimination
+        """
+        self.y_raw = y
+        self.mvr = MultiVariableRegression(
+            X,
+            y,
+            saved_reg=saved_reg,
+            var_names=var_names,
+            back_elim=back_elim,
+            back_elim_p=back_elim_p,
+        )
+        ControlChart.__init__(
+            self,
+            self.mvr.residuals,
+            std=std,
+            ucl_limit=ucl_limit,
+            lcl_limit=lcl_limit,
+            x=x,
+        )
+
+
 class PCA(sklearn_PCA):
     """Hotelling's t-squared statistic for multivariate hypothesis testing"""
 
